// Copyright (c) The Geekeey Authors
// SPDX-License-Identifier: EUPL-1.2

namespace Geekeey.Extensions.Result.Tests;

[TestFixture]
internal sealed class PreludeTests
{
	[Test]
	public void I_can_try_with_success_value_and_get_a_success_result()
	{
		var result = Try(() => 2);

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.True);
			Assert.That(result.Value, Is.EqualTo(2));
		});
	}

	[Test]
	public void I_can_try_with_throwing_exception_and_get_a_failure_result()
	{
		var result = Try<int>(() => throw new CustomTestException());

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.Error, Is.InstanceOf<ExceptionError>());
			var instance = result.Error as ExceptionError;
			Assert.That(instance?.Exception, Is.InstanceOf<CustomTestException>());
		});
	}

	[Test]
	public async Task I_can_try_with_async_success_value_and_get_a_success_result()
	{
		var result = await TryAsync(() => Task.FromResult(2));

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.True);
			Assert.That(result.Value, Is.EqualTo(2));
		});
	}

	[Test]
	public async Task I_can_try_with_async_throwing_exception_and_get_a_failure_result()
	{
		var result = await TryAsync(Task<int> () => throw new CustomTestException());

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.Error, Is.InstanceOf<ExceptionError>());
			var instance = result.Error as ExceptionError;
			Assert.That(instance?.Exception, Is.InstanceOf<CustomTestException>());
		});
	}

	[Test]
	public async Task I_can_try_with_async_await_throwing_exception_and_get_a_failure_result()
	{
		var result = await TryAsync(async Task<int> () =>
		{
			await Task.CompletedTask;
			throw new CustomTestException();
		});

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.Error, Is.InstanceOf<ExceptionError>());
			var instance = result.Error as ExceptionError;
			Assert.That(instance?.Exception, Is.InstanceOf<CustomTestException>());
		});
	}

	[Test]
	public async Task I_can_try_with_async_success_value_and_get_a_success_result_of_type_ValueTask()
	{
		var result = await TryAsync(() => ValueTask.FromResult(2));

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.True);
			Assert.That(result.Value, Is.EqualTo(2));
		});
	}

	[Test]
	public async Task I_can_try_with_async_throwing_exception_and_get_a_failure_result_of_type_ValueTask()
	{
		var result = await TryAsync(ValueTask<int> () => throw new CustomTestException());

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.Error, Is.InstanceOf<ExceptionError>());
			var instance = result.Error as ExceptionError;
			Assert.That(instance?.Exception, Is.InstanceOf<CustomTestException>());
		});
	}

	[Test]
	public async Task I_can_try_with_async_await_throwing_exception_and_get_a_failure_result_of_type_ValueTask()
	{
		var result = await TryAsync(async ValueTask<int> () =>
		{
			await Task.CompletedTask;
			throw new CustomTestException();
		});

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.Error, Is.InstanceOf<ExceptionError>());
			var instance = result.Error as ExceptionError;
			Assert.That(instance?.Exception, Is.InstanceOf<CustomTestException>());
		});
	}
}