// Copyright (c) The Geekeey Authors
// SPDX-License-Identifier: EUPL-1.2

namespace Geekeey.Extensions.Result.Tests;

[TestFixture]
internal sealed class ResultConversionTests
{
	[Test]
	public void I_can_implicitly_convert_from_value_and_get_success()
	{
		var result = Success(2);

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.True);
			Assert.That(result.IsFailure, Is.False);
			Assert.That(result.Value, Is.EqualTo(2));
		});
	}

	[Test]
	public void I_can_implicitly_convert_from_error_and_get_failure()
	{
		var error = new CustomTestError();
		var result = Failure<int>(error);

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.IsFailure, Is.True);
			Assert.That(result.Error, Is.InstanceOf<CustomTestError>());
		});
	}

	[Test]
	public void I_can_unwrap_and_get_value_for_success()
	{
		var result = Success(2);
		var value = result.Unwrap();

		Assert.That(value, Is.EqualTo(2));
	}

	[Test]
	public void I_can_unwrap_and_get_exception_for_failure()
	{
		var result = Failure<int>("error");

		Assert.That(() => result.Unwrap(), Throws.InstanceOf<UnwrapException>());
	}

	[Test]
	public void I_can_explicitly_convert_and_get_value_for_success()
	{
		var result = Success(2);
		var value = (int)result;

		Assert.That(value, Is.EqualTo(2));
	}

	[Test]
	public void I_can_explicitly_convert_and_get_exception_for_failure()
	{
		var result = Failure<int>("error");

		Assert.That(() => (int)result, Throws.InstanceOf<UnwrapException>());
	}
}