// Copyright (c) The Geekeey Authors
// SPDX-License-Identifier: EUPL-1.2

namespace Geekeey.Extensions.Result.Tests;

[TestFixture]
internal sealed class ResultTests
{
	[Test]
	public void I_can_create_new_success_result_from_t()
	{
		var result = new Result<int>(1);

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.True);
			Assert.That(result.IsFailure, Is.False);
			Assert.That(result.Value, Is.Not.EqualTo(default(int)));
			Assert.That(result.Error, Is.Null);
		});
	}

	[Test]
	public void I_can_create_new_failure_result_from_error()
	{
		var result = new Result<int>(new CustomTestError());

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.IsFailure, Is.True);
			Assert.That(result.Value, Is.EqualTo(default(int)));
			Assert.That(result.Error, Is.InstanceOf<CustomTestError>());
		});
	}

	[Test]
	public void I_can_distinguish_default_result_from_created()
	{
		var result = default(Result<int>);

		Assert.Multiple(() =>
		{
			Assert.That(result.IsSuccess, Is.False);
			Assert.That(result.IsFailure, Is.True);
			Assert.That(result.Value, Is.EqualTo(default(int)));
			Assert.That(result.Error, Is.EqualTo(Error.DefaultValueError));
		});
	}

	[Test]
	public void I_can_to_string_success_result_value()
	{
		Result<int> result = 2;

		Assert.That(result.ToString(), Is.EqualTo("Success { 2 }"));
	}

	[Test]
	public void I_can_to_string_failure_result_value()
	{
		Result<int> result = new StringError("error");

		Assert.That(result.ToString(), Is.EqualTo("Failure { error }"));
	}
}